# tt_cpoint_at_intersection.rb
#-----------------------------------------------------------------------------
# Version: 1.0.1
# Compatible: SketchUp 7
#-----------------------------------------------------------------------------
#
# Places a Contruction Point where selected edges intersects with selected faces.
#
#-----------------------------------------------------------------------------
#
# Changelog
# 1.0.0 - 30.06.2009
#       * Initial Release
#
# 1.0.1 - 30.06.2009
#       * Fixed a bug where CPoint where added off the face.
#
#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------

module TT_CPoint_Intersect
	def self.add_cpoint
		model = Sketchup.active_model
		sel = model.selection
		
		edges = []
		faces = []
		
		sel.each { |e|
			edges << e if e.is_a?(Sketchup::Edge)
			faces << e if e.is_a?(Sketchup::Face)
		}
		
		edges.each { |edge|
			faces.each { |face|
				intersect = Geom.intersect_line_plane(edge.line, face.plane)
				next if intersect.nil?
				# Check if the intersection is on the face
				next if (face.classify_point(intersect) & 0b111) == 0 # Returns 0 if value is 0 or 8+
				# Check if the intersection is on the edge
				p1 = intersect - edge.start.position
				p2 = intersect - edge.end.position
				next if p1.z < 0 && p2.z < 0
				next if p1.z > 0 && p2.z > 0
				model.active_entities.add_cpoint(intersect)
			}
		}
	end
end

#-----------------------------------------------------------------------------
# Add some menu items to access this
unless file_loaded?('tt_cpoint_at_intersection.rb')
    plugins_menu = UI.menu('Plugins')
    plugins_menu.add_item('CPoints at Intersection') { TT_CPoint_Intersect::add_cpoint() }
end
#-----------------------------------------------------------------------------
file_loaded('tt_cpoint_at_intersection.rb')
#-----------------------------------------------------------------------------